/*	  Copyright (C) 2001  Sony Computer Entertainment America
       	  
       	  This file is subject to the terms and conditions of the GNU General
       	  Public License Version 2. See the file "COPYING" in the main
       	  directory of this archive for more details.                               */

#ifndef ps2s_cpu_vector_h
#define ps2s_cpu_vector_h

#include <stdio.h>

// #include "ps2s/vector.h"

/********************************************
 * cpu_vec_3 (or cpu_vec_xyz)
 */

class cpu_vec_3
{
   public:
      float x, y, z;

      inline cpu_vec_3() { }

      inline cpu_vec_3( const cpu_vec_3 &vec ) {
	 x = vec.x; y = vec.y; z = vec.z;
      };

      inline cpu_vec_3( float _x, float _y, float _z ) {
	 x = _x; y = _y; z = _z;
      }

      inline void set ( float _x, float _y, float _z ) {
	 x = _x; y = _y; z = _z;
      }

//        inline void set ( const vec128_t vec )
//        {
//  	 vec128_t temp0;
//  	 asm volatile (
//  	    ".if %A0 \n "
      
//  	    "mtsab  $0, 4                # get ready to shift right 4 bytes  \n"
//  	    "move    _x, _vec            # x = value.x  \n"
//  	    "qfsrv  _temp0, $0, _vec    # temp0 = value >> 32  \n"
//  	    "move    _y, _temp0          # y = value.y \n"
//  	    "qfsrv  _temp0, $0, _temp0  # temp0 >>= 32 \n"
//  	    "move    _z, _temp0          # z = value.z \n"
      
//  	    ".else \n"
        
//  	    "mtsab  $0, 4                # get ready to shift right 4 bytes  \n"
//  	    "mtc1    _vec, _x            # x = value.x  \n"
//  	    "qfsrv  _temp0, $0, _vec    # temp0 = value >> 32  \n"
//  	    "mtc1    _temp0, _y          # y = value.y \n"
//  	    "qfsrv  _temp0, $0, _temp0  # temp0 >>= 32 \n"
//  	    "mtc1    _temp0, _z          # z = value.z \n"
        
//  	    ".endif \n"
//  	    : "=&r,f _x" (x), "=&r,f _y" (y), "=&r,f _z" (z), "=&r,&r _temp0" (temp0)
//  	    : "r,r _vec" (vec)
//  	    );
//        }

//        explicit inline cpu_vec_3( const vec_3 vec ) { set(vec.vec128); }
//        explicit inline cpu_vec_3( const vector_t vec ) { set(vec.vec128); }
//        explicit inline cpu_vec_3( const point_t pnt ) { set(pnt.vec128); }

      inline float& operator [] (const int index) {
	 if (index == 0) return x;
	 else if (index == 1) return y;
	 else if (index == 2) return z;
	 else {
	    printf("cpu_vec_3::operator[]: invalid index = %d\n", index);
	    exit(-1);
	    return x;
	 }
      }

      // use () to access individual elements when you don't need write access
      inline float operator () (const int index) const {
	 if (index == 0) return x;
	 else if (index == 1) return y;
	 else if (index == 2) return z;
	 else {
	    printf("cpu_vec_3::operator(): invalid index = %d\n", index);
	    exit(-1);
	    return x;
	 }
      }

      inline cpu_vec_3 & operator = (const cpu_vec_3 &vec) {
	 x = vec.x; y = vec.y; z = vec.z;
	 return *this;
      }

      inline bool operator == (const cpu_vec_3 &rhs) {
	 return ( x == rhs.x
		  && y == rhs.y
		  && z == rhs.z );
      }

      inline bool operator != (const cpu_vec_3 &rhs) {
	 return ( x != rhs.x
		  || y != rhs.y
		  || z != rhs.z );
      }

      inline cpu_vec_3 operator + (const cpu_vec_3 &vec);
      inline cpu_vec_3 operator - (const cpu_vec_3 &vec);
      inline cpu_vec_3 operator - ();
      inline cpu_vec_3 operator * (float scalar);
      inline float dot(const cpu_vec_3 &vec) const;
      inline cpu_vec_3 cross(const cpu_vec_3 &vec);
      inline cpu_vec_3 normalized();
      inline void normalize() {
	 *this = this->normalized();
      }

      void print() const {
	 printf("(%f %f %f *)\n", x, y, z);
      }
};

typedef cpu_vec_3 cpu_vec_xyz;

/********************************************
 * cpu_vec_4 (or cpu_vec_xyzw)
 */

class cpu_vec_4
{
   public:
      float x, y, z, w;

      inline cpu_vec_4() { }

      inline cpu_vec_4( const cpu_vec_4 &vec ) {
	 x = vec.x; y = vec.y; z = vec.z; w = vec.w;
      };

      inline cpu_vec_4( float _x, float _y, float _z, float _w ) {
	 x = _x; y = _y; z = _z; w = _w;
      }

      inline void set ( float _x, float _y, float _z, float _w ) {
	 x = _x; y = _y; z = _z; w = _w;
      }

//        inline void set ( const vec128_t vec )
//        {
//  	 vec128_t temp0;
//  	 asm volatile (
      
//  	    ".if %A0 \n "
      
//  	    "mtsab  $0, 4                # get ready to shift right 4 bytes  \n"
//  	    "move    _x, _vec            # x = value.x  \n"
//  	    "qfsrv  _temp0, $0, _vec    # temp0 = value >> 32  \n"
//  	    "move    _y, _temp0          # y = value.y \n"
//  	    "qfsrv  _temp0, $0, _temp0  # temp0 >>= 32 \n"
//  	    "move    _z, _temp0          # z = value.z \n"
//  	    "qfsrv  _temp0, $0, _temp0  # temp0 >>= 32 \n"
//  	    "move    _w, _temp0          # w = value.w \n"
      
//  	    ".else \n"
        
//  	    "mtsab  $0, 4                # get ready to shift right 4 bytes  \n"
//  	    "mtc1    _vec, _x            # x = value.x  \n"
//  	    "qfsrv  _temp0, $0, _vec    # temp0 = value >> 32  \n"
//  	    "mtc1    _temp0, _y          # y = value.y \n"
//  	    "qfsrv  _temp0, $0, _temp0  # temp0 >>= 32 \n"
//  	    "mtc1    _temp0, _z          # z = value.z \n"
//  	    "qfsrv  _temp0, $0, _temp0  # temp0 >>= 32 \n"
//  	    "mtc1    _temp0, _w          # w = value.w \n"
        
//  	    ".endif \n"
//  	    : "=&r,f _x" (x), "=&r,f _y" (y), "=&r,f _z" (z), "=&r,f _w" (w), "=&r,&r _temp0" (temp0)
//  	    : "r,r _vec" (vec)
//  	    );
//        }

//        explicit inline cpu_vec_4( const vec_4 vec ) { set(vec.vec128); }
//        explicit inline cpu_vec_4( const vector_t vec ) { set(vec.vec128); w = 0.0f; }
//        explicit inline cpu_vec_4( const point_t vec ) { set(vec.vec128); w = 1.0f; }

      inline float& operator [] (const int index) {
	 if (index == 0) return x;
	 else if (index == 1) return y;
	 else if (index == 2) return z;
	 else if (index == 3) return w;
	 else {
	    printf("cpu_vec_4::operator[]: invalid index = %d\n", index);
	    exit(-1);
	    return x;
	 }
      }

      // use () to access individual elements when you don't need write access
      inline float operator () (const int index) const {
	 if (index == 0) return x;
	 else if (index == 1) return y;
	 else if (index == 2) return z;
	 else if (index == 3) return w;
	 else {
	    printf("cpu_vec_4::operator(): invalid index = %d\n", index);
	    exit(-1);
	    return x;
	 }
      }

      inline cpu_vec_4& operator = (const cpu_vec_4 &vec) {
	 x = vec.x; y = vec.y; z = vec.z; w = vec.w;
	 return *this;
      }

      inline cpu_vec_4& operator = (const cpu_vec_3 &vec) {
	 x = vec.x; y = vec.y; z = vec.z;
	 return *this;
      }

      inline bool operator == (const cpu_vec_4 &rhs) {
	 return ( x == rhs.x
		  && y == rhs.y
		  && z == rhs.z
		  && w == rhs.w );
      }

      inline bool operator != (const cpu_vec_4 &rhs) {
	 return ( x != rhs.x
		  || y != rhs.y
		  || z != rhs.z
		  || w != rhs.w );
      }

      inline cpu_vec_4 operator + (const cpu_vec_4 &vec);
      inline cpu_vec_4 operator - (const cpu_vec_4 &vec);
      inline cpu_vec_4 operator - ();
      inline cpu_vec_4 operator * (const cpu_vec_4 &vec);
      inline cpu_vec_4 operator * (float scalar);
      inline float dot(const cpu_vec_4 &vec) const;
      inline float length() const;
      inline cpu_vec_4 normalized();
      inline const cpu_vec_4& normalize();

      void print() const {
	 printf("(%f %f %f %f)\n", x, y, z, w);
      }
};

typedef cpu_vec_4 cpu_vec_xyzw;

/********************************************
 * cpu_vec_3 methods
 */

inline
cpu_vec_3
cpu_vec_3::operator + (const cpu_vec_3 &vec) return result;
{
#if 1

   result.x = x + vec.x;
   result.y = y + vec.y;
   result.z = z + vec.z;

#else

   asm ( " ### cpu_vec_3 + cpu_vec_3 ### \n"
	 "add.s rx, v0x, v1x \n"
	 "add.s ry, v0y, v1y \n"
	 "add.s rz, v0z, v1z \n"
	 : "=&f rx" (result.x), "=&f ry" (result.y), "=&f rz" (result.z)
	 : "f v0x" (x), "f v0y" (y), "f v0z" (z),
	 "f v1x" (vec.x), "f v1y" (vec.y), "f v1z" (vec.z)
      );

#endif
}

inline
cpu_vec_3
cpu_vec_3::operator - (const cpu_vec_3 &vec) return result;
{
#if 1

   result.x = x - vec.x;
   result.y = y - vec.y;
   result.z = z - vec.z;

#else

   asm ( " ### cpu_vec_3 - cpu_vec_3 ### \n"
	 "sub.s rx, v0x, v1x \n"
	 "sub.s ry, v0y, v1y \n"
	 "sub.s rz, v0z, v1z \n"
	 : "=&f rx" (result.x), "=&f ry" (result.y), "=&f rz" (result.z)
	 : "f v0x" (x), "f v0y" (y), "f v0z" (z),
	 "f v1x" (vec.x), "f v1y" (vec.y), "f v1z" (vec.z)
      );

#endif

}

inline
cpu_vec_3
cpu_vec_3::operator - () return result;
{
#if 1

   result.x = -x;
   result.y = -y;
   result.z = -z;

#else

   asm ( " ### -cpu_vec_3 ### \n"
	 "neg.s rx, vx \n"
	 "neg.s ry, vy \n"
	 "neg.s rz, vz \n"
	 : "=&f rx" (result.x), "=&f ry" (result.y), "=&f rz" (result.z)
	 : "f vx" (x), "f vy" (y), "f vz" (z)
      );

#endif

}

inline
cpu_vec_3
cpu_vec_3::operator * (float scalar) return result;
{

#if 1

   result.x = x * scalar;
   result.y = y * scalar;
   result.z = z * scalar;

#else

   asm ( " ### cpu_vec_3 * scalar ### \n"
	 "mul.s rx, vx, scalar \n"
	 "mul.s ry, vy, scalar \n"
	 "mul.s rz, vz, scalar \n"
	 : "=&f rx" (result.x), "=&f ry" (result.y), "=&f rz" (result.z)
	 : "f vx" (x), "f vy" (y), "f vz" (z), "f scalar" (scalar)
      );

#endif

}

inline
float
cpu_vec_3::dot(const cpu_vec_3 &vec) const
{
   float result;

   asm ( " ### cpu_vec_3 dot product ### \n"
	 "mula.s %1, %4 \n"
	 "madda.s %2, %5 \n"
	 "madd.s %0, %3, %6 \n"
	 : "=&f" (result)
	 : "f" (x), "f" (y), "f" (z),
	 "f" (vec.x), "f" (vec.y), "f" (vec.z)
      );

   return result;
}

inline
cpu_vec_3
cpu_vec_3::cross(const cpu_vec_3 &vec) return result;
{
   asm ( " ### cpu_vec_3 cross product ### \n"
	 "mula.s %4, %8 \n"
	 "msub.s %0, %5, %7 \n"
	 "mula.s %5, %6 \n"
	 "msub.s %1, %3, %8 \n"
	 "mula.s %3, %7 \n"
	 "msub.s %2, %4, %6 \n"
	 : "=&f" (result.x), "=&f" (result.y), "=&f" (result.z)
	 : "f" (x), "f" (y), "f" (z),
	 "f" (vec.x), "f" (vec.y), "f" (vec.z)
      );
}

inline
cpu_vec_3
cpu_vec_3::normalized() return result;
{
   float lengthInv = 1.0f/sqrtf(this->dot(*this));
   result = *this * lengthInv;
}

/********************************************
 * cpu_vec_4 methods
 */

inline
cpu_vec_4
cpu_vec_4::operator + (const cpu_vec_4 &vec) return result;
{
   result.x = x + vec.x;
   result.y = y + vec.y;
   result.z = z + vec.z;
   result.w = w + vec.w;
}

inline
cpu_vec_4
cpu_vec_4::operator - (const cpu_vec_4 &vec) return result;
{
   result.x = x - vec.x;
   result.y = y - vec.y;
   result.z = z - vec.z;
   result.w = w - vec.w;
}

inline
cpu_vec_4
cpu_vec_4::operator - () return result;
{
   result.x = -x;
   result.y = -y;
   result.z = -z;
   result.w = -w;
}

inline
cpu_vec_4
cpu_vec_4::operator * (const cpu_vec_4 &vec) return result;
{
   result.x = x * vec.x;
   result.y = y * vec.y;
   result.z = z * vec.z;
   result.w = w * vec.w;
}

inline
cpu_vec_4
cpu_vec_4::operator * (float scalar) return result;
{
   result.x = x * scalar;
   result.y = y * scalar;
   result.z = z * scalar;
   result.w = w * scalar;
}

inline
float
cpu_vec_4::dot(const cpu_vec_4 &vec) const
{
   float result;

   asm ( " ### cpu_vec_4 dot product ### \n"
	 "mula.s %1, %5 \n"
	 "madda.s %2, %6 \n"
	 "madda.s %3, %7 \n"
	 "madd.s %0, %4, %8 \n"
	 : "=&f" (result)
	 : "f" (x), "f" (y), "f" (z), "f" (w),
	 "f" (vec.x), "f" (vec.y), "f" (vec.z), "f" (vec.w)
      );

   return result;
}

inline float
cpu_vec_4::length() const
{
   return sqrtf(this->dot(*this));
}

inline
cpu_vec_4
cpu_vec_4::normalized() return result;
{
   float lengthInv = 1.0f/length();
   result = *this * lengthInv;
}

const cpu_vec_4&
cpu_vec_4::normalize()
{
   *this = normalized();
   return *this;
}

/********************************************
 * initialization of vu0 vectors with cpu_vectors
 */

//  inline
//  vec_xyz::vec_xyz( const cpu_vec_3 &rhs ) {

//     vec128_t temp;
//     asm("### construct vec_3 with cpu_vec_3 ### \n"
//         "pcpyld _temp, _y, _x \n"
//         "ppacw _result, _z, _temp \n"
//         : "=&r _result" (vec128), "=&r _temp" (temp)
//         : "r _x" (rhs.x), "r _y" (rhs.y), "r _z" (rhs.z)
//        );
//  }

//  inline
//  vector_t::vector_t( const cpu_vec_3 &rhs ) {

//     vec128_t temp;
//     asm("### construct vector_t with cpu_vec_3 ### \n"
//         "pcpyld _temp, _y, _x \n"
//         "ppacw _result, _z, _temp \n"
//         : "=&r _result" (vec128), "=&r _temp" (temp)
//         : "r _x" (rhs.x), "r _y" (rhs.y), "r _z" (rhs.z)
//        );
//  }

//  inline
//  point_t::point_t( const cpu_vec_3 &rhs ) {

//     vec128_t temp;
//     asm("### construct point_t with cpu_vec_3 ### \n"
//         "pcpyld _temp, _y, _x \n"
//         "ppacw _result, _z, _temp \n"
//         : "=&r _result" (vec128), "=&r _temp" (temp)
//         : "r _x" (rhs.x), "r _y" (rhs.y), "r _z" (rhs.z)
//        );    
//  }

//  inline
//  vec_xyzw::vec_xyzw( const cpu_vec_4 &rhs ) {
//     vec128_t tempxz, tempyw;

//     asm("### construct vec_4 with cpu_vec_4 ### \n"
//         "pextlw tempxz, _z, _x \n"
//         "pextlw tempyw, _w, _y \n"
//         "pextlw result, tempyw, tempxz\n"
//         : "=&r result" (vec128), "=&r tempxz" (tempxz), "=&r tempyw" (tempyw)
//         : "r _x" (rhs.x), "r _y" (rhs.y), "r _z" (rhs.z), "r _w" (rhs.w)
//        );
//  }

#endif // ps2s_cpu_vector_h
