/*	  Copyright (C) 2001  Sony Computer Entertainment America
       	  
       	  This file is subject to the terms and conditions of the GNU General
       	  Public License Version 2. See the file "COPYING" in the main
       	  directory of this archive for more details.                               */

#ifndef ps2s_eetimer_h
#define ps2s_eetimer_h

// #include <eeregs.h>

typedef struct {
	unsigned COUNT: 16;	// Counter Value
	unsigned p0   : 16;
} tT_COUNT;

typedef struct {
	unsigned CLKS: 2;	// Clock Select
	unsigned GATE: 1;	// Gate Function
	unsigned GATS: 1;	// Gate Select
	unsigned GATM: 2;	// Gate mode
	unsigned ZRET: 1;	// Zero Return
	unsigned CUE : 1;	// Count Up Enable
	unsigned CMPE: 1;	// Interrupt by compare
	unsigned OVFE: 1;	// Interrupt by overflow
	unsigned EQUF: 1;	// Equal Flag
	unsigned OVFF: 1;	// overflow Flag
	unsigned p0  :20;
} tT_MODE;

class CEETimer
{
   public:
      typedef enum { BusClock = 0, BusClock_16th, BusClock_256th, HBlank } tResolution;
      static const unsigned int TicksPerFrame[4] = { 2500000, 156250, 9760, 262 };

      CEETimer( volatile unsigned int* mode, volatile unsigned int* count );

      // accessors

      inline unsigned int GetTicks( void ) const {	return *pCount; }
      inline tResolution GetResolution( void ) const { return Resolution; }
      inline unsigned int GetTicksPerFrame( void ) const { return TicksPerFrame[Resolution]; }
		
      // mutators

      inline void SetResolution( tResolution res ) {
	 Resolution = res;
	 tMode.CLKS = (unsigned int)res;
      }
      inline void Start( void ) {
	 tMode.CUE = 1;
	 tCount.COUNT = 0;
	 *pCount = *((unsigned int*)&tCount);
	 *pMode = *((unsigned int*)&tMode);
      }
      inline unsigned int Stop( void ) {
	 tMode.CUE = 0;
	 *pMode = *((unsigned int*)&tMode);
	 return GetTicks();
      }


   protected:
      tT_MODE tMode;
      tT_COUNT tCount;
      tResolution Resolution;

      volatile unsigned int* pMode;
      volatile unsigned int* pCount;
};

class PerfTimer
{
   public:
      inline static void Start( void ) {
	 asm __volatile__ ("
				mtc0	$0, $9
				sync.p
			");
      }
      inline static unsigned int GetTicks( void ) {
	 unsigned int ret;

	 asm __volatile__ ("
            mfc0	%0, $9
            sync.p
         " : "=r" (ret) );

	 return ret;
      }
      inline static unsigned int Stop( void ) {
	 return GetTicks();
      }
};

#endif // ps2s_eetimer_h
