/*	  Copyright (C) 2001  Sony Computer Entertainment America
       	  
       	  This file is subject to the terms and conditions of the GNU General
       	  Public License Version 2. See the file "COPYING" in the main
       	  directory of this archive for more details.                               */

#ifndef ps2s_imagepackets_h
#define ps2s_imagepackets_h

#include "ps2s/packet.h"
#include "ps2s/core.h"
#include "ps2s/gs.h"

class CTexture;

/********************************************
 * CImageUploadPkt
 */

class CImageUploadPkt : protected CVifSCDmaPacket
{
      friend class CTexture;

   public:
      CImageUploadPkt( tU128* imagePtr, tU32 w, tU32 h, GS::tPSM psm, tU32 gsBufWidth = 0, tU32 gsWordAddress = 0 );
      virtual ~CImageUploadPkt( void ) {}

      void* operator new( size_t size ) {
	 return Core::New16(size);
      }

      // call these before trying to xfer the image!
      virtual void SetGsAddr( tU32 gsMemWordAddress ) {
	 gsrBitBltBuf.DBP = gsMemWordAddress/64;
      }
      void SetGsBufferWidth( tU32 gsBufWidth ) {
	 mAssert( gsBufWidth >= 64 );
	 gsrBitBltBuf.DBW = gsBufWidth / 64;
      }

      // if you use this constructor you need to call these functions before
      // trying to xfer the image:  SetImage, SetGsBufferWidth, SetGsAddr
      CImageUploadPkt( void );

      void SetImage( tU128* imagePtr, tU32 w, tU32 h, GS::tPSM psm ) {
	 mAssert( ((tU32)imagePtr & 0xf) == 0 );
	 pImage = imagePtr;
	 gsrBitBltBuf.DPSM = psm;
	 gsrTrxReg.RRW = w; gsrTrxReg.RRH = h;

	 BuildXferTags();
      }

      void Reset() { CVifSCDmaPacket::Reset(); }

      void Send( bool waitForEnd = false, bool flushCache = true ) {
	 CSCDmaPacket::Send( waitForEnd, flushCache );
      }

      void Send( CSCDmaPacket& packet );
      void Send( CVifSCDmaPacket& packet );

   protected:

   private:
      // gs packet to setup the texture image transfer
      tU128				FirstDmaTag;
      tU128				SecondDmaTag; // for linux
      tGifTag			ImageXferSettingsGifTag;
      ps2_gsreg_bitbltbuf	gsrBitBltBuf;	tU64 BitBltBufAddr;
      ps2_gsreg_trxpos		gsrTrxPos;	tU64 TrxPosAddr;
      ps2_gsreg_trxreg		gsrTrxReg;	tU64 TrxRegAddr;
      ps2_gsreg_trxdir		gsrTrxDir;      tU64 TrxDirAddr;
      tU128			RestOfPacket[45];

      tU32			uiNumXferGSRegs;
      tU128*			pImage;

      void InitCommon( void );
      void BuildXferTags( void );
} __attribute__ (( aligned(16) ));

/********************************************
 * CClutUploadPkt
 */

class CClutUploadPkt : protected CImageUploadPkt
{
      friend class CTexture;

   public:
      CClutUploadPkt( tU32 *clutPtr, tU32 gsMemWordAddr ) {
	 SetGsBufferWidth( 64 );
	 SetGsAddr( gsMemWordAddr );
	 SetClut( clutPtr );
      }

      CClutUploadPkt( void ) { 
	 SetGsBufferWidth( 64 );
      }
      virtual ~CClutUploadPkt( void ) {}

      void* operator new( size_t size ) {
	 return Core::New16(size);
      }

      void SetGsAddr( tU32 gsMemWordAddress ) {
	 CImageUploadPkt::SetGsAddr( gsMemWordAddress );
      }
      void SetClut( tU32* clutPtr ) {
	 SetImage( (tU128*)clutPtr, 16, 16, GS::kPsm32 );
      }
      void Reset() { CVifSCDmaPacket::Reset(); }
      void Send( bool waitForEnd = false, bool flushCache = true ) {
	 CImageUploadPkt::Send( waitForEnd, flushCache );
      }

      inline void Send( CSCDmaPacket& packet ) { CImageUploadPkt::Send(packet); }
      inline void Send( CVifSCDmaPacket& packet ) { CImageUploadPkt::Send(packet); }
} __attribute__ (( aligned(16) ));

#endif // ps2s_imagepackets_h
